Package: lazyeval
Version: 0.2.3
Title: Lazy (Non-Standard) Evaluation
Description: An alternative approach to non-standard evaluation using
    formulas. Provides a full implementation of LISP style 'quasiquotation',
    making it easier to generate code with other code.
Authors@R: c(
    person("Hadley", "Wickham", ,"hadley@rstudio.com", c("aut", "cre")),
    person("RStudio", role = "cph")
    )
License: GPL-3
Depends: R (>= 3.1.0)
Imports: rlang
Suggests: knitr, rmarkdown (>= 0.2.65), testthat, covr
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/build/compilation-database: true
NeedsCompilation: yes
Packaged: 2026-04-03 09:37:04 UTC; lionel
Author: Hadley Wickham [aut, cre],
  RStudio [cph]
Maintainer: Hadley Wickham <hadley@rstudio.com>
Repository: CRAN
Date/Publication: 2026-04-04 05:10:53 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 05:36:05 UTC; unix
Archs: lazyeval.so.dSYM
