% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazy-as.R
\name{all_dots}
\alias{all_dots}
\title{Combine explicit and implicit dots.}
\usage{
all_dots(.dots, ..., all_named = FALSE)
}
\arguments{
\item{.dots}{A list of lazy objects}

\item{...}{Individual lazy objects}

\item{all_named}{If \code{TRUE}, uses \code{\link{auto_name}} to ensure
every component has a name.}
}
\value{
A \code{\link{lazy_dots}}
}
\description{
Combine explicit and implicit dots.
}
\keyword{internal}
