% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{is_formula}
\alias{is_formula}
\title{Is object a formula?}
\usage{
is_formula(x)
}
\arguments{
\item{x}{Object to test}
}
\description{
Is object a formula?
}
\examples{
is_formula(~ 10)
is_formula(10)
}
